
MyEventHandler = EVENT:New()
EscortMonitor = EVENTHANDLER:New()


spawnedGroups = {}
local escortGroups = {}
local menuEscortRequest = {}
local escortRequestMenus = {}
local escortMenus = {}



function AddEscortRequestMenu(group)
    if not group then
        env.info("AddEscortRequestMenu: Group is nil, cannot create menu.")
        return
    end
    local groupName = group:GetName()
    escortRequestMenus[groupName] = MENU_GROUP_COMMAND:New(group, "Request Escort", nil, EscortClientGroup, group)
end

function EnableEscortRequestMenu(group)
    if not group then
        env.info("EnableEscortRequestMenu: Group is nil, cannot enable menu.")
        return
    end
    local groupName = group:GetName()
    if menuEscortRequest[groupName] then
        menuEscortRequest[groupName]:Remove()
    end
end

function RequestEscort(group)
    EscortClientGroup(group)
    local groupName = group:GetName()
    if menuEscortRequest[groupName] then
        menuEscortRequest[groupName]:Remove()
        menuEscortRequest[groupName] = nil
    end
end

function FindEscortTemplateWithAlias(alias)
    local escortSpawn = SPAWN:NewWithAlias("EscortA10", alias)
    return escortSpawn
end

function RemoveRequestEscortMenu(group)
    local groupName = group:GetName()
    if escortRequestMenus[groupName] then
        escortRequestMenus[groupName]:Remove()
        escortRequestMenus[groupName] = nil
    end
end

function EscortClientGroup(clientGroup)
    local groupName = clientGroup:GetName()
    local alias = groupName .. "_Escort"
    local escortSpawn = FindEscortTemplateWithAlias(alias)
    local escortGroup = escortSpawn:Spawn()

    if not escortGroup then
        env.info("Failed to spawn escort group for " .. groupName)
        return
    end

    local clientPosition = clientGroup:GetPointVec3()
    local clientHeading = clientGroup:GetHeading()

    
    local distanceBehindMeters = 3000
    local offsetX = math.cos(math.rad(clientHeading)) * distanceBehindMeters
    local offsetZ = math.sin(math.rad(clientHeading)) * distanceBehindMeters

    local escortSpawnPosition = {
        x = clientPosition.x - offsetX,
        y = clientPosition.y + 3000,
        z = clientPosition.z - offsetZ
    }

    local escortCoord = COORDINATE:NewFromVec3(escortSpawnPosition)

    local escortAuftrag = AUFTRAG:NewESCORT(clientGroup, {x=-100, y=3048, z=300}, 40, {"Air"})
	
	
	

    escortGroup = FLIGHTGROUP:New(escortGroup:GetName())  
    escortAuftrag:SetEPLRS(true)
    escortGroup:Teleport(escortCoord)
    escortGroups[groupName] = escortGroup
	

    local dcsGroup = escortGroup:GetDCSObject()

    if dcsGroup then

        local controllableGroup = CONTROLLABLE:New(dcsGroup)

        
        controllableGroup:CommandSetUnlimitedFuel(true, 0)
    else
        env.info("DCS group object not found.")
    end

    MESSAGE:New("Your escort is on route.", 10):ToGroup(clientGroup)

    escortGroup:AddMission(escortAuftrag)
    RemoveRequestEscortMenu(clientGroup)
    AddEscortMenu(clientGroup)
end


function AddEscortMenu(group)
    if not group then
        env.info("AddEscortMenu: Group is nil, cannot create menu.")
        return
    end

    local menuEscort = MENU_GROUP:New(group, "Escort")
    MENU_GROUP_COMMAND:New(group, "Start Orbit", menuEscort, EscortOrbit, group)
    MENU_GROUP_COMMAND:New(group, "Racetrack, North to South", menuEscort, RaceTrackNorthSouth, group)
	MENU_GROUP_COMMAND:New(group, "Racetrack, East to West", menuEscort, RaceTrackWestEast, group)
    MENU_GROUP_COMMAND:New(group, "Rejoin", menuEscort, EscortRejoin, group)
	MENU_GROUP_COMMAND:New(group, "Escort RTB", menuEscort, EscortAbort, group)
end

function RemoveEscortMenu(group)
    local groupName = group:GetName()
    if escortMenus[groupName] then
        escortMenus[groupName]:Remove()
        escortMenus[groupName] = nil
        env.info("Escort menu for " .. groupName .. " has been removed.")
    end
end

function EscortOrbit(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        local clientCoord = group:GetPointVec3()
        local orbitAuftrag = AUFTRAG:NewORBIT_CIRCLE(clientCoord, 25000, 300)
        
        escortGroup:CancelAllMissions()
        escortGroup:AddMission(orbitAuftrag)
        
        
        MESSAGE:New("Escort is orbiting around your current position.", 10):ToGroup(group)
    else
        MESSAGE:New("No active escort found.", 10):ToGroup(group)
    end
end

function RaceTrackNorthSouth(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        local clientCoord = group:GetPointVec3()
		
        local RaceTracknorthtosouthAuftrag = AUFTRAG:NewPATROL_RACETRACK(clientCoord, 25000, 300, 0, 10)
        
        escortGroup:CancelAllMissions()
        escortGroup:AddMission(RaceTracknorthtosouthAuftrag)
        
        
        MESSAGE:New("Escort is setting up a 10 NM racetrack north to south.", 10):ToGroup(group)
    else
        MESSAGE:New("No active escort found.", 10):ToGroup(group)
    end
end
function RaceTrackWestEast(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        local clientCoord = group:GetPointVec3()
		
        local RaceTrackwesttosouthAuftrag = AUFTRAG:NewPATROL_RACETRACK(clientCoord, 25000, 300, 090, 10)
        
        escortGroup:CancelAllMissions()
        escortGroup:AddMission(RaceTrackwesttosouthAuftrag)
        
        
        MESSAGE:New("Escort is setting up a 10 NM racetrack East to West.", 10):ToGroup(group)
    else
        MESSAGE:New("No active escort found.", 10):ToGroup(group)
    end
end

function EscortRejoin(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
        escortGroup:CancelAllMissions()

        local escortAuftrag = AUFTRAG:NewESCORT(group, {x=0, y=3048, z=-3000}, 40, {"Air"})
        escortGroup:AddMission(escortAuftrag)
        
        
        MESSAGE:New("Escort is rejoining your formation.", 10):ToGroup(group)
    else
        MESSAGE:New("No active escort found.", 10):ToGroup(group)
    end
end


function EscortAbort(group)
    local escortGroup = escortGroups[group:GetName()]
    if escortGroup then
                
        escortGroup:CancelAllMissions()
        
        
        MESSAGE:New("Escort is RTB", 10):ToGroup(group)
    else
        MESSAGE:New("No active escort found.", 10):ToGroup(group)
    end
end

function MyEventHandler:OnEventTakeoff(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local groupSet = SET_GROUP:New():FilterPrefixes("A-10C"):FilterAlive():FilterStart()
        
        groupSet:ForEachGroup(function(group)
            if group:GetUnit(1):GetID() == playerUnit:GetID() then
                local groupName = group:GetName()

                if not spawnedGroups[groupName] then
                    spawnedGroups[groupName] = {
                        playerName = EventData.IniPlayerName,
                        escortGroups = {},
                        menuEscortRequest = nil
                    }

                    env.info("Player " .. EventData.IniPlayerName .. " has taken off in group: " .. groupName)
                    
                    MESSAGE:New("Escort is available to be requested from the radio menu, " .. EventData.IniPlayerName .. ".", 10, ""):ToUnit(playerUnit)

                    AddEscortRequestMenu(group)
                    menuEscortRequest[groupName] = spawnedGroups[groupName].menuEscortRequest
                end
            end
        end)
    end
end
function MyEventHandler:OnEventPlayerLeaveUnit(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local playerGroup = playerUnit:GetGroup()

        if playerGroup then
            local groupName = playerGroup:GetName()
            local escortGroup = escortGroups[groupName]

            if escortGroup then
                escortGroup:Destroy()
                escortGroups[groupName] = nil
                env.info("Escort group for " .. groupName .. " has been destroyed because the player left the unit.")
            end
        end
    end
end

function MyEventHandler:OnEventLand(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local playerGroup = playerUnit:GetGroup()

        if playerGroup then
            local groupName = playerGroup:GetName()
            local escortGroup = escortGroups[groupName]

            if escortGroup then
                escortGroup:Destroy()
                escortGroups[groupName] = nil
                env.info("Escort group for " .. groupName .. " has been destroyed because the player landed.")
            end
        end
    end
end
function MyEventHandler:OnEventPilotDead(EventData)
    if EventData.IniUnit and EventData.IniPlayerName then
        local playerUnit = EventData.IniUnit
        local playerGroup = playerUnit:GetGroup()

        if playerGroup then
            local groupName = playerGroup:GetName()
            local escortGroup = escortGroups[groupName]

            if escortGroup then
                escortGroup:Destroy()
                escortGroups[groupName] = nil
                env.info("Escort group for " .. groupName .. " has been destroyed because the player landed.")
            end
        end
    end
end



function EscortMonitor:OnEventDead(EventData)
    if EventData.IniGroup then
        local escortGroupName = EventData.IniGroup:GetName()
        
        if escortGroups[escortGroupName] then
            env.info("Escort group " .. escortGroupName .. " has been destroyed.")

			escortGroups[escortGroupName] = nil

            
            local clientGroup = EventData.IniGroup
            MESSAGE:New("Your escort group has been destroyed\nLand and you'll have a new Escort group ready upon take off.", 10):ToGroup(clientGroup)
			
			RemoveEscortMenu(clientGroup)
        end
    end
end





MyEventHandler:HandleEvent(EVENTS.PlayerLeaveUnit)
MyEventHandler:HandleEvent(EVENTS.Land)
MyEventHandler:HandleEvent(EVENTS.Takeoff)
MyEventHandler:HandleEvent(EVENTS.PilotDead)
EscortMonitor:HandleEvent(EVENTS.Dead)
